# Copyright (C) 2018 Semicon Networks
# Author: Hans Roh <hansroh@gmail.com>

__version__ = 2

import os
import sys
import subprocess
from getpass import getpass
from hashlib import md5
import os, struct
from ..termcolor import tc
from .. import ifaces
import time
ENCRYPTED = None
LICENSE_FILE = None
POSITION = 0

def configure (key, name = '.license'):
    global ENCRYPTED, LICENSE_FILE, POSITION
    ENCRYPTED = md5 (key.encode ()).hexdigest ()
    POSITION = (sum ([ord (c) for c in ENCRYPTED]) * 1978524790) % 1008
    LICENSE_FILE = os.path.join (os.path.dirname (os.path.join (os.getcwd (), sys.argv [0])), name)

def hide_file (filename):
    import win32file
    import win32con
    import win32api
    flags = win32file.GetFileAttributesW(filename)
    win32file.SetFileAttributes(filename,
        win32con.FILE_ATTRIBUTE_HIDDEN | flags)

def check_license ():
    global ENCRYPTED, LICENSE_FILE
    assert ENCRYPTED and LICENSE_FILE, 'call license.configure first'

    if not os.path.isfile (LICENSE_FILE):
        return "cannot find license"
    try:
        hwid = ifaces.get_hwid ().lower ()
    except subprocess.CalledProcessError:
        return "cannot get hardware ID, maybe run wiht root permission"
    key = md5 ('{}:{}'.format (ENCRYPTED, hwid).encode ()).digest ()
    with open (LICENSE_FILE, 'rb') as f:
        data = f.read ()
    ver = struct.unpack ('H', data [:2])
    s = sum (struct.unpack ('BBB', data [2:5])) + 5 + POSITION
    if data [s:s + 16] != key:
        return "invalid license"
    a, b = struct.unpack ('II', data [s+16:s+24])
    if int (time.time ()) > a + b:
        return "license expired"

def create_license (uuid = None):
    global ENCRYPTED, LICENSE_FILE
    assert ENCRYPTED and LICENSE_FILE, 'call license.configure first'

    if os.path.isfile (LICENSE_FILE):
        with open (LICENSE_FILE, 'rb') as f:
            data = f.read ()
        lns = struct.unpack ('HBBB', data [:5])
        data = data [5:]
        ver = lns [0]
        name = data [:lns [1]].decode (); data = data [lns [1]:]
        org = data [:lns [2]].decode (); data = data [lns [2]:]
        contact = data [:lns [3]].decode (); data = data [lns [3]:]

        print (check_license ())
        print (tc.ok ("\nLicense Information\n"))
        print (tc.echo ("  Organization:"), org)
        print (tc.echo ("  Point of Contact Name:"), name)
        print (tc.echo ("  Email/Phone Number:"), contact)
        print (tc.echo ("\n  License Status:"), check_license () and tc.error ('INVALID') or tc.ok ('VALID'))

        if input ('\nDo you want to replace to new license? (n/y) ') != 'y':
            sys.exit ()

    print ()
    org, name, contact = None, None, None
    while not org:
        org = input ('Enter Organization Name: ')
        if len (org.encode ()) > 64:
            print ('  too long, should be within 64 bytes')
            org = None
    while not name:
        name = input ('Enter Point of Contact Name: ')
        if len (name.encode ()) > 64:
            print ('  too long, should be within 64 bytes')
            name = None
    while not contact:
        contact = input ('Enter Email or Phone Number: ')
        if len (contact.encode ()) > 32:
            print ('  too long, should be within 32 bytes')
            contact = None

    name = name.encode ()
    org = org.encode ()
    contact = contact.encode ()

    premble = struct.pack ('H', __version__) + struct.pack ('B', len (name)) + struct.pack ('B', len (org)) + struct.pack ('B', len (contact)) + name + org + contact
    for i in range (3):
        password = getpass('\nEnter License Permit Key: ')
        try:
            password, period = password.split ('/', 1)
        except ValueError:
            period = 0
        passed = False
        try:
            period = int (period)
        except ValueError:
            print ("  incorrect key, try again...")
            continue
        if md5 (password.encode ()).hexdigest () == ENCRYPTED:
            passed = True
            break
        print ("  incorrect key, try again...")

    if not passed:
        return "incorrect key"

    try:
        hwid = uuid or ifaces.get_hwid ()
        hwid = hwid.lower ()
    except subprocess.CalledProcessError:
        return "cannot get hardware ID, maybe run wiht root permission"
    key = md5 ('{}:{}'.format (ENCRYPTED, hwid).encode ()).digest () + struct.pack ('I', period * 24 * 3600) + struct.pack ('I', int (time.time ()))
    data = premble + os.urandom (POSITION) + key + os.urandom (1024 - 20 - POSITION)
    with open (LICENSE_FILE, 'wb') as f:
        f.write (data)
    os.chmod (LICENSE_FILE, 0o644)



# 2016. 1. 10 by Hans Roh hansroh@gmail.com

__version__ = "0.2.13.17"

version_info = tuple (map (lambda x: not x.isdigit () and x or int (x),  __version__.split (".")))

from multiprocessing import Pool
from tqdm import tqdm as tqdm_
import concurrent.futures
from colorama import Fore
from functools import partial
from .termcolor import tc
import subprocess as subprocess_
import asyncio as asyncio_
from asyncio.tasks import Task
import sys
if sys.version_info >= (3, 5):
    from .aio import aio

# for lower version compatibles -------------
from . import annotations as versioning
deco = versioning
class udict: pass
# end of compatibles -----------------------

# abbreviations ----------------------------------------------------
def tqdm (iterable, desc = "", color = None, **karg):
    if color:
        return tqdm_ (iterable, desc, bar_format = "{l_bar}%s{bar}%s{r_bar}" % (getattr (Fore, color.upper ()), Fore.RESET), **karg)
    else:
        return tqdm_ (iterable, desc, **karg)

def tpool (workers = 1):
    return concurrent.futures.ThreadPoolExecutor (max_workers = workers)
threading = tpool

def ppool (workers = 1):
    return concurrent.futures.ProcessPoolExecutor (max_workers = workers)
processing = ppool

def ppool2 (workers = 1):
    return Pool (workers)
processing2 = ppool2

def subprocessing (cmd_or_func, *args):
    if isinstance (cmd_or_func, (list, tuple, str)):
        res = subprocess_.run (cmd_or_func, shell = True, stdout = subprocess_.PIPE, stderr = subprocess_.PIPE)
        out, err = res.stdout.decode ("utf8"), res.stderr.decode ("utf8")
        if err:
            raise RuntimeError (err)
        return out

    with Pool (1) as p:
        return p.apply (cmd_or_func, args)

def waitf (futures, timeout = None):
    if not futures:
        return [], []
    # wait Futures or async Tasks
    if isinstance (futures [0], Task):
        return aio.wait (futures, timeout = timeout)
    else:
        return concurrent.futures.wait (futures, timeout = timeout)

def wait (futures, timeout = None):
    dones, undones = waitf (futures)
    return timeout is None and dones or (dones, undones)

def as_completed (futures):
    return concurrent.futures.as_completed (futures)



# Copyright (C) 2017 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Sample that implements a text client for the Google Assistant Service."""

import os
import logging
import json

import click
import google.auth.transport.grpc
import google.auth.transport.requests
import google.oauth2.credentials

from google.assistant.embedded.v1alpha2 import (
    embedded_assistant_pb2,
    embedded_assistant_pb2_grpc
)

try:
    from . import (
        assistant_helpers,
        browser_helpers,
    )
except (SystemError, ImportError):
    import assistant_helpers
    import browser_helpers


ASSISTANT_API_ENDPOINT = 'embeddedassistant.googleapis.com'
DEFAULT_GRPC_DEADLINE = 60 * 3 + 5
PLAYING = embedded_assistant_pb2.ScreenOutConfig.PLAYING


class SampleTextAssistant(object):
    """Sample Assistant that supports text based conversations.

    Args:
      language_code: language for the conversation.
      device_model_id: identifier of the device model.
      device_id: identifier of the registered device instance.
      display: enable visual display of assistant response.
      channel: authorized gRPC channel for connection to the
        Google Assistant API.
      deadline_sec: gRPC deadline in seconds for Google Assistant API call.
    """

    def __init__(self, language_code, device_model_id, device_id,
                 display, channel, deadline_sec):
        self.language_code = language_code
        self.device_model_id = device_model_id
        self.device_id = device_id
        self.conversation_state = None
        # Force reset of first conversation.
        self.is_new_conversation = True
        self.display = display
        self.assistant = embedded_assistant_pb2_grpc.EmbeddedAssistantStub(
            channel
        )
        self.deadline = deadline_sec

    def __enter__(self):
        return self

    def __exit__(self, etype, e, traceback):
        if e:
            return False

    def assist(self, text_query):
        """Send a text request to the Assistant and playback the response.
        """
        def iter_assist_requests():
            config = embedded_assistant_pb2.AssistConfig(
                audio_out_config=embedded_assistant_pb2.AudioOutConfig(
                    encoding='LINEAR16',
                    sample_rate_hertz=16000,
                    volume_percentage=0,
                ),
                dialog_state_in=embedded_assistant_pb2.DialogStateIn(
                    language_code=self.language_code,
                    conversation_state=self.conversation_state,
                    is_new_conversation=self.is_new_conversation,
                ),
                device_config=embedded_assistant_pb2.DeviceConfig(
                    device_id=self.device_id,
                    device_model_id=self.device_model_id,
                ),
                text_query=text_query,
            )
            # Continue current conversation with later requests.
            self.is_new_conversation = False
            if self.display:
                config.screen_out_config.screen_mode = PLAYING
            req = embedded_assistant_pb2.AssistRequest(config=config)
            assistant_helpers.log_assist_request_without_audio(req)
            yield req

        text_response = None
        html_response = None
        for resp in self.assistant.Assist(iter_assist_requests(),
                                          self.deadline):
            assistant_helpers.log_assist_response_without_audio(resp)
            if resp.screen_out.data:
                html_response = resp.screen_out.data
            if resp.dialog_state_out.conversation_state:
                conversation_state = resp.dialog_state_out.conversation_state
                self.conversation_state = conversation_state
            if resp.dialog_state_out.supplemental_display_text:
                text_response = resp.dialog_state_out.supplemental_display_text
        return text_response, html_response


@click.command()
@click.option('--api-endpoint', default=ASSISTANT_API_ENDPOINT,
              metavar='<api endpoint>', show_default=True,
              help='Address of Google Assistant API service.')
@click.option('--credentials',
              metavar='<credentials>', show_default=True,
              default=os.path.join(click.get_app_dir('google-oauthlib-tool'),
                                   'credentials.json'),
              help='Path to read OAuth2 credentials.')
@click.option('--device-model-id',
              metavar='<device model id>',
              required=True,
              help=(('Unique device model identifier, '
                     'if not specifed, it is read from --device-config')))
@click.option('--device-id',
              metavar='<device id>',
              required=True,
              help=(('Unique registered device instance identifier, '
                     'if not specified, it is read from --device-config, '
                     'if no device_config found: a new device is registered '
                     'using a unique id and a new device config is saved')))
@click.option('--lang', show_default=True,
              metavar='<language code>',
              default='en-US',
              help='Language code of the Assistant')
@click.option('--display', is_flag=True, default=False,
              help='Enable visual display of Assistant responses in HTML.')
@click.option('--verbose', '-v', is_flag=True, default=False,
              help='Verbose logging.')
@click.option('--grpc-deadline', default=DEFAULT_GRPC_DEADLINE,
              metavar='<grpc deadline>', show_default=True,
              help='gRPC deadline in seconds')
def main(api_endpoint, credentials,
         device_model_id, device_id, lang, display, verbose,
         grpc_deadline, *args, **kwargs):
    # Setup logging.
    logging.basicConfig(level=logging.DEBUG if verbose else logging.INFO)

    # Load OAuth 2.0 credentials.
    try:
        with open(credentials, 'r') as f:
            credentials = google.oauth2.credentials.Credentials(token=None,
                                                                **json.load(f))
            http_request = google.auth.transport.requests.Request()
            credentials.refresh(http_request)
    except Exception as e:
        logging.error('Error loading credentials: %s', e)
        logging.error('Run google-oauthlib-tool to initialize '
                      'new OAuth 2.0 credentials.')
        return

    # Create an authorized gRPC channel.
    grpc_channel = google.auth.transport.grpc.secure_authorized_channel(
        credentials, http_request, api_endpoint)
    logging.info('Connecting to %s', api_endpoint)

    with SampleTextAssistant(lang, device_model_id, device_id, display,
                             grpc_channel, grpc_deadline) as assistant:
        while True:
            query = click.prompt('')
            click.echo('<you> %s' % query)
            response_text, response_html = assistant.assist(text_query=query)
            if display and response_html:
                system_browser = browser_helpers.system_browser
                system_browser.display(response_html)
            if response_text:
                click.echo('<@assistant> %s' % response_text)


if __name__ == '__main__':
    main()



import sys
PY_MAJOR_VERSION = sys.version_info.major

def is_str_like (thing):
	if PY_MAJOR_VERSION == 2 and type (thing) is unicode:
		return True
	return type (thing) is str

def is_encodable (thing):
	if PY_MAJOR_VERSION == 2:
		if type (thing) is unicode: 
			return True
	return type (thing) is str

def is_decodable (thing):
	if PY_MAJOR_VERSION == 2:
		if type (thing) is str:
			return True
	return type (thing) is bytes




import telegram

class Telegram:
	def __init__ (self, token):
		self.token = token
		self.bot = telegram.Bot(token = token)
		
	def show_messages (self):
		updates = self.bot.getUpdates()
		for u in updates:
			print(u.message)
	
	def updates (self):
		return self.bot.getUpdates()
		
	def send (self, msg, chat_id = None):
		self.bot.sendMessage (
			chat_id or self.updates ()[-1].message.chat.id, 
			msg
		)



from selenium.webdriver.common.by import By
import re

By.TEXT_IN_TAG = "text in tag"
By.CSS_TEXT = "css text"

def get_finding_string (params):
	what = ""
	if not params:
		return By.CSS_SELECTOR, ""
	if params [0] in "\"'":
		quot = params [0]								
		pos = 1
		while 1:
			s = params.find (quot, pos)
			if s ==- 1:
				raise AssertionError ("Quotes Not Matched In ADU Rule")
			if params [s-1] == "\\":
				pos = s + 1
			else:
				break
		what = params [1:s]
		params = params [s + 1:].strip ()
	
	s = params.find (" by ")
	if s == -1:
		by = "CSS_SELECTOR"
		if what.startswith ("/"):
			by = "XPATH"
	else:
		by = params [s + 4:].strip ()
		params = params [:s]
		
	if not what:
		what = params.strip ()
	
	try:
		by = getattr (By, by)
	except AttributeError:
		raise AssertionError	 ("Unknown By Object '%s' In ADU Rule" % by)
					
	return by, what

def make_detector (by, what):
	from .nops import nops as p
	
	if by == By.XPATH:
		fmethod, fobject = p.by_xpath, what
	elif by == By.TEXT_IN_TAG:
		fmethod, fobject = p.by_tint, what
	elif by == By.CSS_TEXT:
		fmethod, fobject = p.by_csstext, what			
	elif by == By.LINK_TEXT:
		fmethod, fobject = p.by_lt, what
	elif by == By.PARTIAL_LINK_TEXT:
		fmethod, fobject = p.by_plt, what			
	elif by == By.TAG_NAME:	
		fmethod, fobject = p.by_tag, what
	else:
		if by == By.ID:
			css = "#" + what
		elif by == By.CLASS_NAME:	
			css = "." + what
		elif by == By.NAME:	
			css = '[name="%s"]' % what
		elif by == By.CSS_SELECTOR:
			css = what
		else:
			raise AssertionError ("Unknown By Method")			
		if css.find (":data") != -1:
			fmethod, fobject = p.by_css_data, css				
		else:	
			fmethod, fobject = p.by_css, css
							
	return fmethod, fobject

def get_parser_method (param):
	return make_detector (*get_finding_string (param))
get_finder = get_parser_method

RX_TAGS = re.compile (
	(
		"(img|a|table|div|span|font|small|b|i|u|strong|"
		"tr|td|th|p|blockquote|ul|ol|dl|li|dt|dl|dd|button|"
		"input|form|textarea|options|select)"
		"($|[^a-z])"
	)
)

RX_UF = re.compile (r"(^|[^a-z])([gf]\.[a-zA-Z][_a-zA-Z0-9]*\s*\()")

def get_extractor (params):
	ffrom = None
	temp = params.split (" from ", 1)
	if len (temp) == 2:
		ffrom = temp [0].strip ()
		params = temp [1]
		return (ffrom,) + (get_finder (params))	
	
	if RX_UF.search (params):		
		return (params.strip (), None, None)
		
	if params [0] in ".#/" or params.find ('rf-') != -1 or params.find (':data') != -1 or RX_TAGS.match (params):
		# definitly html eleemt
		return ('data',) + (get_finder (params))
		
	return (params.strip (), None, None)


RX_VARNAME = re.compile (r'\(\s*([a-z]+)?\s*[),.]')	
def get_var (ctx):
	var = ctx
	if ctx.find ("g.") != -1 or ctx.find ("f.") != -1:
		try:
			var = RX_VARNAME.findall (ctx)[0]
		except IndexError:	
			var = ctx	
	return var
	


import boto3
import botocore
import paramiko

if __name__ == "__main__":
    key = paramiko.RSAKey.from_private_key_file("path/to/mykey.pem")
    client = paramiko.SSHClient()
    client.set_missing_host_key_policy(paramiko.AutoAddPolicy())

    client.connect(hostname=instance_ip, username="ubuntu", pkey=key)
    stdin, stdout, stderr = client.exec_command(cmd)
    print (stdout.read())
    client.close()



from django.contrib import admin
from django.utils.safestring import mark_safe
from django.forms import Widget
import mimetypes

def set_title (title):
    admin.site.site_title = title
    admin.site.site_header = "{}".format (title)
    admin.site.index_title = "{} Management Console".format (title)

def get_type (path):
    return mimetypes.guess_type (os.path.basename (path))[0]

def ImageWidget (width = 360):
    class _ImageWidget(Widget):
        def render(self, name, value, **noneed):
            return value and mark_safe ('<img src="{}" width="{}">'.format (value, width)) or 'No Image'
    return _ImageWidget

class LinkWidget(Widget):
    def render(self, name, value, **noneed):
        return value and mark_safe ('<a href="{}">{}</a> [<a href="{}" target="_blank">새창</a>]'.format (value, value, value)) or 'No Image'

def VideoWidget (video_width = 320, video_height = 240):
    class _VideoWidget(Widget):
        def render(self, name, value, **noneed):
            return value and mark_safe (
                '<video width="{}" height="{}" controls><source src="{}" type="{}"></video>'.format (
                    video_width, video_height, value, get_type (value)
                )
            ) or 'No Video'
    return _VideoWidget

class AudioWidget(Widget):
    def render(self, name, value, **noneed):
        return value and mark_safe ('<audio controls><source src="{}" type="{}"></audio>'.format (
            value, get_type (value)
            )
        ) or 'No Audio'


class ModelAdmin (admin.ModelAdmin):
    image_width = 320
    video_width = 320
    enable_add = True
    enable_delete = True
    enable_change = True
    def has_add_permission(self, request, obj=None):
        return self.enable_add

    def has_delete_permission(self, request, obj=None):
        return self.enable_delete

    def has_change_permission(self, request, obj=None):
        return self.enable_change

    def formfield_for_dbfield (self, db_field, request, **kwargs):
        if 'widget' not in kwargs:
            if db_field.name.endswith ('image'):
                kwargs ['widget'] = ImageWidget (self.image_width)
                return db_field.formfield(**kwargs)
            elif db_field.name.endswith ('video'):
                kwargs ['widget'] = VideoWidget (self.video_width)
                return db_field.formfield(**kwargs)
            elif db_field.name.endswith ('audio'):
                kwargs ['widget'] = AudioWidget
                return db_field.formfield(**kwargs)
            elif db_field.name.endswith ('url'):
                kwargs ['widget'] = LinkWidget
                return db_field.formfield(**kwargs)
        return super ().formfield_for_dbfield (db_field, request, **kwargs)


from .houndify import *
import re

CLIENT_ID, CLIENT_KEY = None, None

def set_credential (client_id, client_key):
    global CLIENT_ID, CLIENT_KEY

    CLIENT_ID, CLIENT_KEY = client_id, client_key

RE_PRONOUNCE = re.compile (r"\(/.+?/\)")
def get_answer (question, user = 'test_user'):
    global CLIENT_ID, CLIENT_KEY

    client = houndify.TextHoundClient (CLIENT_ID, CLIENT_KEY, user)
    response = client.query(question)
    try:
        best = response ['AllResults'][0]
    except (KeyError, IndexError):
        return {}
    else:
        subject = best ['WrittenResponse']
        answer = RE_PRONOUNCE.sub ('', best ['WrittenResponseLong'])
        return dict (
            best = {}, answer = answer, subject = subject
        )



import requests
import re
import json

ETRI_OPEN_API_URL = "http://aiopen.etri.re.kr:8000/WikiQA"
RE_KANJI = re.compile (r'[㐀-䶵一-鿋豈-頻]')

def get_answer (question):
    from . import ACCESS_KEY

    requestJson = {
        "access_key": ACCESS_KEY,
        "argument": {
            "question": question,
            "type": "ENGINE_TYPE"
        }
    }
    response = requests.post (ETRI_OPEN_API_URL, json.dumps(requestJson), headers={"Content-Type": "application/json; charset=UTF-8"})
    result = response.json ()
    try:
        best = result ['return_object']['WiKiInfo']
        subject = best ['IRInfo'][0]['wiki_title']
        answer = best ['AnswerInfo'][0]['answer']
        answer = RE_KANJI.sub ('', answer)
    except (IndexError, KeyError):
        return {}
    else:
        return dict (
            best = best, answer = answer, subject = subject
        )



from ... import pathtool
import os
from .speech import (
    get_speech,
    get_translate
)

GCP_DIR = os.path.join (os.environ ['HOME'], '.gcp'); pathtool.mkdir (GCP_DIR)
DEFAULT_CREDENTIAL = os.path.join (GCP_DIR, 'credential.json')

def set_credential (path = None):
    global DEFAULT_CREDENTIAL

    if path is None:
        path = DEFAULT_CREDENTIAL
    assert os.path.isfile (path), 'credential file not found: {}'.format (path)
    os.environ ['GOOGLE_APPLICATION_CREDENTIALS'] = path

if os.path.isfile (DEFAULT_CREDENTIAL):
    set_credential ()



try:
    import asyncio as asyncio_
except ImportError:
    aio = None
else:
    # asyncio abbreviations --------------------------------------------
    class aio:
        @classmethod
        def loop (cls, func, *args, **karg):
            loop = asyncio_.get_event_loop ()
            loop.run_until_complete (func (*args, **karg))  
        
        @classmethod
        def wait (cls, futures, timeout = None):
            return asyncio_.wait (futures, timeout = timeout)
        
        @classmethod
        def submit (cls, func, *args, **karg):
            return asyncio_.ensure_future(func (*args, **karg)) # Future
        
        @classmethod
        async def map (cls, func, iterable):
            futures = [cls.submit (func,  item) for item in iterable]
            return await asyncio_.gather (*futures)
    
        @classmethod
        async def thread (cls, func, *args, **karg):
            loop = asyncio_.get_event_loop()  
            return await loop.run_in_executor (None, func, *args, **karg) # Future



import urllib.request, urllib.parse, urllib.error
import re
import time
from ...lib import logger as loggerfactory
import os
import random
from ...lib import pathtool
import math

class ProxyServerList:
	def __init__ (self, logger = None):
		self.logger = logger
		self.proxies = []
		self.proxy_reused = 0
	
	def __len__ (self):
		return len (self.proxies)
			
	def pop (self):
		if not self.proxies:
			self.load ()		
		proxy = self.proxies.pop (0)
		if self.logger: self.logger ("[info] %d proxies was remained" % len (self.proxies))
		return proxy		
	
	def getr (self):
		if not self.proxies:
			self.load ()		
		proxy = random.choice (self.proxies)
		return proxy		
	
	def putr (self, proxy):
		index = random.randrange (len (self.proxies))
		self.insert (index, proxy)
	
	get = getr
	put = putr
	
	def append (self, proxy):
		self.proxies.append (proxy)
	
	def insert (self,index, proxy):
		self.proxies.insert (index, proxy)	
		
	def load (self, path = None):
		if path is None:
			path = os.path.join (os.environ ["systemroot"], "sharedproxy.dat")
		f = open (path)
		for line in f:
			h, p = line.strip ().split (":", 1)
			self.proxies.append ((h, int (p)))
		f.close ()
		random.shuffle (self.proxies)
		if self.logger: self.logger ("[info] %d proxies loaded" % len (self.proxies))
	
	def save (self, path):
		f = open (path, "w")
		for proxy in self.proxies:
			f.write ("%s:%s\n" % proxy)
		f.close ()
		
	def duplicate (self, output, path = None):
		if path is None:
			path = os.path.join (os.environ ["systemroot"], "sharedproxy.dat")
		self.proxy_reused += 1
		f = open (path)
		data = f.read ()
		f.close ()
		f = open (output, "w")
		f.write (data)
		f.close ()	
	
		
if __name__ == "__main__":
	f = ProxyServerList ()
	f.load ()



#!/usr/bin/env python3
import houndify
import sys
import time
import wave


CLIENT_ID = sys.argv[1]
CLIENT_KEY = sys.argv[2]
AUDIO_FILE = sys.argv[3]

BUFFER_SIZE = 256


audio = wave.open(AUDIO_FILE)
if audio.getsampwidth() != 2:
  print("%s: wrong sample width (must be 16-bit)" % fname)
  sys.exit()
if audio.getframerate() != 8000 and audio.getframerate() != 16000:
  print("%s: unsupported sampling frequency (must be either 8 or 16 khz)" % fname)
  sys.exit()
if audio.getnchannels() != 1:
  print("%s: must be single channel (mono)" % fname)
  sys.exit()


audio_size = audio.getnframes() * audio.getsampwidth()
audio_duration = audio.getnframes() / audio.getframerate()
chunk_duration = BUFFER_SIZE * audio_duration / audio_size


#
# Simplest HoundListener; just print out what we receive.
# You can use these callbacks to interact with your UI.
#
class MyListener(houndify.HoundListener):
  def onPartialTranscript(self, transcript):
    print("Partial transcript: " + transcript)
  def onFinalResponse(self, response):
    print("Final response: " + str(response))
  def onError(self, err):
    print("Error: " + str(err))


client = houndify.StreamingHoundClient(CLIENT_ID, CLIENT_KEY, "test_user", enableVAD=False)
client.setLocation(37.388309, -121.973968)
client.setSampleRate(audio.getframerate())

# # Uncomment the lines below to see an example of using a custom
# # grammar for matching.  Use the file 'turnthelightson.wav' to try it.
# clientMatches = [ {
#   "Expression" : '([1/100 ("can"|"could"|"will"|"would")."you"].[1/10 "please"].("turn"|"switch"|(1/100 "flip"))."on".["the"].("light"|"lights").[1/20 "for"."me"].[1/20 "please"])|([1/100 ("can"|"could"|"will"|"would")."you"].[1/10 "please"].[100 ("turn"|"switch"|(1/100 "flip"))].["the"].("light"|"lights")."on".[1/20 "for"."me"].[1/20 "please"])|((("i".("want"|"like"))|((("i".["would"])|("i\'d")).("like"|"want"))).["the"].("light"|"lights").["turned"|"switched"|("to"."go")|(1/100"flipped")]."on".[1/20"please"])"',
#   "Result" : { "Intent" : "TURN_LIGHT_ON" },
#   "SpokenResponse" : "Ok, I\'m turning the lights on.",
#   "SpokenResponseLong" : "Ok, I\'m turning the lights on.",
#   "WrittenResponse" : "Ok, I\'m turning the lights on.",
#   "WrittenResponseLong" : "Ok, I\'m turning the lights on."
# } ]
#
# client.setHoundRequestInfo('ClientMatches', clientMatches)


client.start(MyListener())

while True:
  chunk_start = time.time()

  samples = audio.readframes(BUFFER_SIZE)
  if len(samples) == 0: break
  if client.fill(samples): break

  # # Uncomment the line below to simulate real-time request
  # time.sleep(chunk_duration - time.time() + chunk_start) 

result = client.finish() # returns either final response or error




from subprocess import PIPE, Popen

class Piped:
    def __init__(self, cmd):
        self.exe_args = cmd
        self.proc = None

    def start(self):
        self.proc = Popen(self.exe_args, stdin=PIPE, stdout=PIPE)

    def stop(self):
        if self.proc:
            self.proc.kill ()
            self.proc = None

    def communicate (self, input):
        self.proc.stdin.write (input)
        self.proc.stdin.flush ()
        return self.proc.stdout.readline ()



from Crypto import Random
from Crypto.Cipher import AES as AESCipher
import base64
from hashlib import md5

def trans (key):
    return md5 (key.encode ()).hexdigest ()

class AES:
    BLCOK_SIZE = 16

    @classmethod
    def encrypt (cls, message, passphrase):
        passphrase = trans (passphrase)
        IV = Random.new ().read (cls.BLCOK_SIZE)
        aes = AESCipher.new (passphrase, AESCipher.MODE_CFB, IV)
        return base64.b64encode (IV + aes.encrypt (message))

    @classmethod
    def decrypt (cls, encrypted, passphrase):
        passphrase = trans (passphrase)
        encrypted = base64.b64decode (encrypted)
        IV = encrypted [:cls.BLCOK_SIZE]
        aes = AESCipher.new (passphrase, AESCipher.MODE_CFB, IV)
        return aes.decrypt (encrypted [cls.BLCOK_SIZE:])


from . import asyncore
import multiprocessing
import time
import sys
import os
from . import pathtool
import types
import codecs
from .termcolor import tc

PY_MAJOR_VERSION = sys.version_info.major

def traceback (multirows = False):
	(file, fun, line), t, v, tbinfo = asyncore.compact_traceback()
	try: v = str (v)
	except:	v = repr (v)
	if multirows:
		line = "%s\n%s Traceback: %s" % (t, v, tbinfo)
		line = trace ().replace ("] [", "\n  - ")
		line = line.replace ("Traceback: [", "\n  -----------\n  + Traceback\n  ===========\n  - ")
		line = line [:-1] + "\n  -----------"
	else:
		line = "%s %s Traceback: %s" % (t, v, tbinfo)
	return line
trace = traceback

def now (detail = 1):
	if detail: return time.strftime("%Y.%m.%d %H:%M:%S", time.localtime(time.time()))
	else: return time.strftime("%Y%m%d", time.localtime(time.time()))

def ago (t):
	diff = time.time () - t
	if diff < 3600:
		return "%2.1f minutes ago" % (diff/60)
	elif diff < 3600	 * 24:
		return "%2.1f hours ago" % (diff/3600)
	elif diff < 3600	 * 72:
		return "%2.1f days ago" % (diff/(3600 * 24))
	return time.strftime ("%Y-%m-%d", time.localtime (t))


class base_logger:
	def __init__(self, out, cacheline = 100, flushnow = 0):
		self.out = out
		self.cacheline = cacheline
		self.flushnow = flushnow
		self.lock = multiprocessing.Lock()
		self.filter = []
		self.__cache = []

	def set_filter (self, *types):
		self.filter = types

	def __call__(self, line, type="info", name=""):
		return self.log (line, type, name)

	def write (self, line, type="", name=""):
		return self.log (line, type, name)

	def cleanup (self):
		self.close ()

	def flush (self):
		try:
			self.out.flush ()
		except:
			pass

	def cache (self, line):
		if self.cacheline:
			self.__cache.insert (0, line)
			self.__cache = self.__cache [:self.cacheline]

	def tag (self, type, name):
		try:
			name = str (name)
		except UnicodeEncodeError:
			pass

		tag = ""
		if type:
			tag = "[%s%s] " % (type, name and ":" + name or "")
		return tag

	def _writeln (self, line):
		with self.lock:
			try:
				self.out.write (line)
			except UnicodeEncodeError:
				self.out.write (repr (line.encode ("utf8")))
			if self.flushnow:
				self.flush ()
			self.cache (line)
		return line

	def log (self, line, type = "info", name = ""):
		line = str (line).strip ()
		line = "%s %s%s\n" % (now(), self.tag (type, name), line)
		if self.filter and type not in self.filter:
			return line
		return self._writeln (line)

	def close (self):
		self.out.close ()

	def trace (self, name = ''):
		return self.log (trace (True), "expt", name)
	traceback = trace

	def read (self):
		return self.__cache

class screen_logger (base_logger):
	def __init__ (self, cacheline = 200, flushnow = 1, colored = True):
		base_logger.__init__(self, sys.stdout, cacheline, flushnow)
		self.colored = colored

	def close (self):
		pass

	def log (self, line, type = "info", name = ""):
		line = str (line).strip ()
		if line and line [0] == "[":
			e = line.find ("]")
			if e != -1:
				type = line [1:e]
				line = line [e + 1:].strip ()

		basetype = type.split (":", 1) [0]
		if self.filter and basetype not in self.filter:
			return line

		if not self.colored:
			line = "{} {}{}\n".format (now(), self.tag (type, name), line)
		else:
			try:
				type_color = getattr (tc, basetype)
			except AttributeError:
				type_color = tc.default
			line = "{} {}{}\n".format (tc.primary (now()), type_color (self.tag (type, name)), line)
		return self._writeln (line)

class null_logger (screen_logger):
	def log (self, line, type = "info", name = ""):
		pass


class pipe_logger (screen_logger):
	def __init__ (self):
		screen_logger.__init__(self, 0, 1)

	def log (self, line, type = "info", name = ""):
		# do not write datetime
		if isinstance (line, str):
			line = line.encode ("ascii", "ignore")

		line = str (line).strip ()
		line = "%s%s\n" % (self.tag (type, name), line)

		if self.filter and type not in self.filter:
			return line

		self.lock.acquire ()
		self.out.write (line)
		if self.flushnow: self.flush ()
		self.cache (line)
		self.lock.release ()
		return line


class rotate_logger (base_logger):
	PRESERVE_MAX = 30
	def __init__(self, base, surfix = '', freq = "daily", cacheline = 200, flushnow = 0):
		self.base = base
		self.surfix = surfix
		self.freq = freq

		pathtool.mkdir (base)
		self.file = "%s/%s.log" % (self.base, self.surfix)

		base_logger.__init__ (self, codecs.open (self.file, "a", "utf8"), cacheline, flushnow)

		self.cv = multiprocessing.Condition (multiprocessing.RLock())
		self.using = 0
		self.numlog = 0
		self.maintern ()
		self.rotate_when = self.get_next_rotate (self.freq)

	def maybe_rotate (self):
		if self.freq and time.time() > self.rotate_when:
			self.rotate()
			self.rotate_when = self.get_next_rotate (self.freq)

	def get_next_rotate (self, freq = "daily"):
		(yr, mo, day, hr, min, sec, wd, jday, dst) = time.localtime(time.time())
		if freq == 'daily':
			return time.mktime((yr,mo,day+1, 0,0,0, 0,0,-1))
			#return time.mktime((yr,mo,day, hr,min,sec+10, 0,0,-1))
		elif freq == 'weekly':
			return time.mktime((yr,mo,day-wd+7, 0,0,0, 0,0,-1))
		elif freq == 'monthly':
			return time.mktime((yr,mo+1,1, 0,0,0, 0,0,-1))
		else:
			return time.mktime((yr,mo,day+1, 0,0,0, 0,0,-1))

	def maintern (self):
		dlogs = []
		for file in os.listdir (self.base):
			if file.find (self.surfix + "-") != 0: continue
			dlogs.append (file)

		if len (dlogs) <= self.PRESERVE_MAX: return
		dlogs.sort ()
		for file in dlogs [:-self.PRESERVE_MAX]:
			try: os.remove (os.path.join (self.base, file))
			except: pass

	def rotate (self):
		self.cv.acquire ()
		try:
			self.out.close ()
			(yr, mo, day, hr, min, sec, wd, jday, dst) = time.localtime(time.time())
			newfile = "%s/%s-%04d%02d%02d.log" % (self.base, self.surfix, yr, mo, day)
			if os.path.isfile (newfile):
				 newfile = "%s/%s-%04d%02d%02d-%02d%02d%02d.log" % (self.base, self.surfix, yr, mo, day,hr, min, sec)
			try:
				os.rename (self.file, newfile)
			except:
				self.out = codecs.open (self.file, "a", "utf8")
			else:
				self.out = codecs.open (self.file, "w", "utf8")
			self.maintern ()

		finally:
			self.cv.release ()

	def close (self):
		self.cv.acquire ()
		while self.using:
			self.cv.wait ()
		try:
			base_logger.close (self)
		finally:
			self.cv.release ()

	def log (self, line, type="info", name=""):
		try:
			line = str (line)
		except UnicodeEncodeError:
			pass

		lline = "%s %s%s\n" % (now(), self.tag (type, name), line)
		if self.filter and type not in self.filter:
			return lline

		self.cv.acquire ()
		self.using = 1
		try:
			self.out.write (lline)
			if self.flushnow: self.flush ()
			self.cache (line)
		except:
			self.out.write ("%s %%s\n", (now(), self.tag (type, name), repr (line)))

		self.using = 0
		self.numlog += 1
		numlog = self.numlog
		self.cv.notify_all ()
		self.cv.release ()

		if numlog % 1000 == 0:
			self.maybe_rotate ()

		return line


class file_logger (rotate_logger):
	def __init__ (self, base, surfix = '', cacheline = 200, flushnow = 0):
		rotate_logger.__init__ (self, base, surfix, cacheline = cacheline, flushnow = flushnow)


class multi_logger (base_logger):
	def __init__(self, loggers = [], cacheline = 100, flushnow = 0):
		self.loggers = []
		for logger in loggers:
			self.loggers.append (logger)

		base_logger.__init__ (self, None, cacheline, flushnow)

	def rotate (self):
		for logger in self.loggers:
			hasattr (logger, 'rotate') and logger.rotate ()

	def log (self, line, type="info", name=""):
		if self.filter and type not in self.filter:
			return line
		for logger in self.loggers:
			_lline = logger.log (line, type, name)
		self.cache (_lline)

	def add_logger (self, logger):
		self.loggers.append (logger)

	def close (self):
		for logger in self.loggers:
			logger.close ()


if __name__ == "__main__":
	l = file_logger ("x:/test/", "test")
	for i in range (300):
		l.write ("asdsadasda")
	l.close ()




import os, sys

#Throws OSError exception (it will be thrown when the process is not allowed
#to switch its effective UID or GID):

def drop_privileges (uid_name = None):
    if os.name == "nt":
        return
    if os.getuid() != 0:
        return

    import pwd, grp
    user_name = os.getenv("SUDO_USER") or uid_name or 'nobody'
    pwnam = pwd.getpwnam(user_name)
    os.setgroups([])
    os.setgid(pwnam.pw_gid)
    os.setuid(pwnam.pw_uid)
    old_umask = os.umask(0o22)

def set_process_name (name):
    if os.name == "posix":
        from setproctitle import setproctitle
        setproctitle (name)

def is_running (pid, cmd = None):
    if cmd is None:
        cmd = os.path.split (sys.argv [0])[1]

    if os.name == "nt":
        import win32process, win32api, win32con, pywintypes
        HAS_WMI = True
        try: import wmi
        except ImportError: HAS_WMI = False

        if pid not in win32process.EnumProcesses ():
            return False

        if HAS_WMI:
            cl = [p.CommandLine for p in wmi.WMI ().Win32_Process () if p.ProcessID == pid]
            if cl and cl [0].find (cmd) != -1:
                return True
            return False

        else:
            try:
                handle = win32api.OpenProcess (win32con.PROCESS_QUERY_INFORMATION | win32con.PROCESS_VM_READ, 0, int (pid))
                exefilename = win32process.GetModuleFileNameEx (handle, 0)
                win32process.GetStartupInfo()
                if exefilename.lower ().find ("python.exe") != -1 or exefilename.lower ().find ("cmd.exe") != -1:
                    return True
            except pywintypes.error:
                # Windows service, Access is denied
                return False

    else:
        proc = "/proc/%s/cmdline" % pid
        if not os.path.isfile (proc):
            return False

        with open (proc) as f:
            exefilename = f.read ()
        if exefilename.find (cmd) != -1:
            return True

    return False

if os.name == "nt":
    import win32pdh
    import win32process
    import win32event
    import pywintypes

    def timeout_execute (cmd, timeout = 0):
        if timeout == 0:
            timeout = win32event.INFINITE

        info  = win32process.CreateProcess(None, cmd, None, None, 0, 0, None, None, win32process.STARTUPINFO())
        subprocess = info [0]

        rc = win32event.WaitForSingleObject (subprocess, timeout)

        if rc == win32event.WAIT_FAILED:
            return -1

        if rc == win32event.WAIT_TIMEOUT:
            try:
                win32process.TerminateProcess (subprocess, 0)
            except pywintypes.error:
                return -3
            return -2

        if rc == win32event.WAIT_OBJECT_0:
            return win32process.GetExitCodeProcess(subprocess)


    def get_child_pid (cpid):
        object = 'Process'
        items, instances = win32pdh.EnumObjectItems(None, None, object,
                                                   win32pdh.PERF_DETAIL_WIZARD)
        instance_dict = {}
        for instance in instances:
            try:
                instance_dict[instance] = instance_dict[instance] + 1
            except KeyError:
                instance_dict[instance] = 0

        processinfos = []
        for instance, max_instances in list(instance_dict.items()):
            for inum in range(max_instances+1):
                processinfo = []
                hq = win32pdh.OpenQuery()
                hcs = []
                for item in ['ID Process', 'Creating Process ID']:
                    path = win32pdh.MakeCounterPath((None,object,instance,None,inum,item))
                    hcs.append(win32pdh.AddCounter(hq,path))
                win32pdh.CollectQueryData(hq)
                processinfo.append (instance[:15].strip ())
                #print "%-15s\t" % (instance[:15]),
                for hc in hcs:
                    type,val=win32pdh.GetFormattedCounterValue(hc,win32pdh.PDH_FMT_LONG)
                    processinfo.append (int (val))
                    #print "%5d" % (val),
                    win32pdh.RemoveCounter(hc)
                #print
                win32pdh.CloseQuery(hq)
                processinfos.append (tuple (processinfo))

        def recusive (cpid):
            pids = []
            for name, pid, ppid in processinfos:
                if ppid == cpid:
                    pids.append (pid)
            for pid in pids:
                pids += recusive (pid)
            return pids

        pids = recusive (cpid)
        return pids

if __name__ == "__main__":
    print(get_child_pid (3172))





from pyssml.AmazonSpeech import AmazonSpeech
import boto3

polly_ = boto3.client('polly')
transcribe_ = boto3.client ('transcribe')

def polly (text, lang = 'ko-KR', voice_id = None):    
    if not voice_id:
        voice_id = lang == "ko-KR" and 'Seoyeon' or 'Joanna'
    response = polly_.synthesize_speech (
        Text = text,
        OutputFormat = 'mp3',
        VoiceId = voice_id,
        TextType = text.find ('</speak>') != -1 and 'ssml' or 'text'
    )
    stream = response.get ("AudioStream")
    return stream.read ()

# transcribe -----------------------------------------------
def upload_s3 (bucket, file):
    s3_.upload_file (file, bucket, file)
    return file

def transcribe (url, lang = 'ko-KR', job_name = 'TranscribeDemo', media_format = 'mp3'):
    transcribe_.start_transcription_job (
        TranscriptionJobName = job_name,
        LanguageCode = lang,
        MediaFormat = media_format,
        Media = {'MediaFileUri': url}
    )
    
